﻿Imports System.Configuration
Imports System.Security.Cryptography
Imports System.Text

Public Class AuthApiWrapper

    'members
    Private Shared apiSecretKey As String = String.Empty
    Private apiToken As String = String.Empty
    Private userName As String = String.Empty
    Private password As String = String.Empty

    Shared Sub New()
        'initialize secretkey
        apiSecretKey = ConfigurationManager.AppSettings("ApiSecretKey")
    End Sub

    Public Sub New()

        'initialize members
        apiToken = ConfigurationManager.AppSettings("ApiToken")
        userName = ConfigurationManager.AppSettings("UserName")
        password = ConfigurationManager.AppSettings("Password")
    End Sub

    'Get user authentication token
    Public Async Function GetAuthenticationToken() As Task(Of String)

        'Populate userauthrequest using signature and epoch seconds
        Dim userRequest = New UserAuthRequest With {
            .User = userName,
            .Password = password,
            .Token = apiToken,
            .RequestDate = GetEpochSeconds(),
            .Signature = GetUserHMACSignature()
        }
        Dim luxSciResponse As LuxSciResponse = Nothing

        'call httpclient
        Using authHttpClient As AuthHttpClient = New AuthHttpClient()
            luxSciResponse = Await authHttpClient.GetAuthenticationCode(userRequest)
        End Using

        'throw error, if the success is not 1. otherwise, return authentication code
        If luxSciResponse.success = 1 Then
            Return luxSciResponse.auth
        Else
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        End If
    End Function

    'get account authentication token
    Public Async Function GetAccountAuthenticationToken() As Task(Of String)
        'populate accountauthrequest using signature and epoch seconds
        Dim userRequest = New AccountAuthRequest With {
            .Token = apiToken,
            .RequestDate = GetEpochSeconds(),
            .Signature = GetAccountHMACSignature()
        }
        Dim luxSciResponse As LuxSciResponse = Nothing

        'call httpclient
        Using authHttpClient As AuthHttpClient = New AuthHttpClient()
            luxSciResponse = Await authHttpClient.GetAuthenticationCode(userRequest)
        End Using

        'return authentication code, if success is 1
        If luxSciResponse.success = 1 Then
            Return luxSciResponse.auth
        Else
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        End If
    End Function

    'Get user signature
    Private Function GetUserHMACSignature() As String
        Return GetHMACSignature(apiToken & vbLf & GetEpochSeconds() & vbLf & userName & vbLf & password & vbLf)
    End Function

    'Get account signature
    Private Function GetAccountHMACSignature() As String
        Return GetHMACSignature(apiToken & vbLf & GetEpochSeconds() & vbLf)
    End Function

    'Get HMACSignature for the given string using secret key
    Public Shared Function GetHMACSignature(ByVal textToSign As String) As String
        Dim hmac = New HMACSHA256(Encoding.UTF8.GetBytes(apiSecretKey))
        Dim hash = hmac.ComputeHash(Encoding.UTF8.GetBytes(textToSign))
        Return BitConverter.ToString(hash).Replace("-", String.Empty)
    End Function

    'Get Epoch seconds
    Private Function GetEpochSeconds() As String
        Dim epoch = DateTime.UtcNow - New DateTime(1970, 1, 1, 0, 0, 0)
        Return (CLng(epoch.TotalSeconds)).ToString()
    End Function

    'Compute hash will be used by both user and account api
    Public Shared Function ComputeSha256Hash(ByVal rawData As String) As String
        Using sha256Hash As SHA256 = SHA256.Create()
            Dim bytes As Byte() = sha256Hash.ComputeHash(Encoding.UTF8.GetBytes(rawData))
            Dim builder As StringBuilder = New StringBuilder()

            For i As Integer = 0 To bytes.Length - 1
                builder.Append(bytes(i).ToString("x2"))
            Next

            Return builder.ToString()
        End Using
    End Function

    Public Shared Function ComputeSha256Hash(ByVal bytes As Byte()) As String
        Using sha256Hash As SHA256 = SHA256.Create()
            Return ToHex(sha256Hash.ComputeHash(bytes), False)
        End Using
    End Function

    Private Shared Function ToHex(ByVal bytes As Byte(), ByVal upperCase As Boolean) As String
        Dim result As StringBuilder = New StringBuilder(bytes.Length * 2)

        For i As Integer = 0 To bytes.Length - 1
            result.Append(bytes(i).ToString(If(upperCase, "X2", "x2")))
        Next

        Return result.ToString()
    End Function
End Class
