﻿Imports System.Configuration
Imports System.IO
Imports System.Net
Imports System.Net.Http
Imports System.Net.Http.Headers
Imports System.Text
Imports Newtonsoft.Json

Public Class UserApiWrapper

    'members
    Private userRequestPath As String = String.Empty
    Private userName As String = String.Empty
    Private hostName As String = String.Empty

    Public Sub New()
        'initialize members
        userRequestPath = ConfigurationManager.AppSettings("UserPath")
        userName = ConfigurationManager.AppSettings("UserName")
        hostName = ConfigurationManager.AppSettings("Host")
    End Sub

    'get user profile
    Public Async Function GetUserProfile() As Task(Of UserProfile)
        'get authentcation code
        Dim authApi As AuthApiWrapper = New AuthApiWrapper()
        Dim authCode As String = Await authApi.GetAuthenticationToken()
        Dim luxSciResponse As LuxSciResponse = Nothing

        'build signature and path
        Dim path As String = userRequestPath & "/" & userName & "/profile"
        Dim signature As String = AuthApiWrapper.GetHMACSignature(authCode & vbLf & "GET" & vbLf & path & vbLf & vbLf & vbLf)

        'call httpclient
        Using userHttpClient As UserHttpClient = New UserHttpClient()
            luxSciResponse = Await userHttpClient.GetUserProfile(path, authCode, signature)
        End Using

        'return user profile json data, if success is 1.
        If luxSciResponse.success = 1 Then
            Return JsonConvert.DeserializeObject(Of UserProfile)(luxSciResponse.data.ToString())
        Else
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        End If
    End Function

    'send mail
    Public Async Function SendMail(ByVal emailRequest As EmailRequest) As Task(Of Boolean)
        'get authentication code
        Dim authApi As AuthApiWrapper = New AuthApiWrapper()
        Dim authCode As String = Await authApi.GetAuthenticationToken()
        Dim luxSciResponse As LuxSciResponse = Nothing

        'build path and signature
        Dim path As String = userRequestPath & "/" & userName & "/email/compose/secureline/send"
        Dim signature As String = AuthApiWrapper.GetHMACSignature(authCode & vbLf & "POST" & vbLf & path & vbLf & vbLf & AuthApiWrapper.ComputeSha256Hash(JsonConvert.SerializeObject(emailRequest).Trim()) & vbLf)

        'call httpclient
        Using userHttpClient As UserHttpClient = New UserHttpClient()
            luxSciResponse = Await userHttpClient.SendMail(path, authCode, signature, emailRequest)
        End Using

        'throw error, if the success is not 1
        If luxSciResponse.success <> 1 Then
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        Else
            Return True
        End If
    End Function

    Public Async Function SendMailWithAttachment(ByVal emailRequest As EmailRequest, ByVal filename As String) As Task(Of Boolean)
        'get authentication code
        Dim authApi As AuthApiWrapper = New AuthApiWrapper()
        Dim authCode As String = Await authApi.GetAuthenticationToken()
        Dim luxSciResponse As LuxSciResponse = Nothing
        Dim boundary As String = Guid.NewGuid().ToString()

        'get file as byte array
        Dim fileContents As Byte() = File.ReadAllBytes(filename)

        'attach the file to the emailrequest
        AttachFile(emailRequest, filename, fileContents)

        'get file and email content
        Dim content = GetMultipartFormContent(emailRequest, filename, boundary, fileContents)

        'build path and signature
        Dim path As String = userRequestPath & "/" & userName & "/email/compose/secureline/send"
        Dim signature As String = AuthApiWrapper.GetHMACSignature(authCode & vbLf & "POST" & vbLf & path & vbLf & vbLf & AuthApiWrapper.ComputeSha256Hash(JsonConvert.SerializeObject(emailRequest).Trim()) & vbLf)

        'call httpclient
        Using userHttpClient As UserHttpClient = New UserHttpClient()
            Dim request = New HttpRequestMessage(HttpMethod.Post, New Uri($"https://{hostName}/{path}")) With {
                .Version = HttpVersion.Version10,
                .content = content
            }
            luxSciResponse = Await userHttpClient.SendMailWithAttachment(path, authCode, signature, request, boundary)
        End Using

        'throw error, if the success is not 1
        If luxSciResponse.success <> 1 Then
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        Else
            Return True
        End If
    End Function

    'attach file to the email request
    Private Sub AttachFile(ByVal emailRequest As EmailRequest, ByVal filename As String, ByVal fileContents As Byte())
        Dim attachments As List(Of Attachment) = New List(Of Attachment)()
        Dim attachement As Attachment = New Attachment()
        attachement.Name = Path.GetFileName(filename)
        attachement.Hash = AuthApiWrapper.ComputeSha256Hash(fileContents)
        attachments.Add(attachement)
        emailRequest.Attachments = attachments.ToArray()
    End Sub

    'get file byte array
    Private Function GetFileByteArrayContent(ByVal filename As String, ByVal fileContents As Byte()) As ByteArrayContent
        Dim fileByteArrayContent As ByteArrayContent = New ByteArrayContent(fileContents)
        fileByteArrayContent.Headers.ContentDisposition = New ContentDispositionHeaderValue("form-data")
        fileByteArrayContent.Headers.ContentDisposition.Name = """files"""
        fileByteArrayContent.Headers.ContentDisposition.FileName = """" & Path.GetFileName(filename) & """"
        fileByteArrayContent.Headers.ContentType = New MediaTypeHeaderValue("application/octet-stream")
        Return fileByteArrayContent
    End Function

    'get mail byte array
    Private Function GetMailByteArrayContent(ByVal emailRequest As EmailRequest) As ByteArrayContent
        Dim mailByteArrayJson As ByteArrayContent = New ByteArrayContent(Encoding.UTF8.GetBytes(JsonConvert.SerializeObject(emailRequest).Trim()))
        mailByteArrayJson.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json")
        mailByteArrayJson.Headers.ContentDisposition = New ContentDispositionHeaderValue("form-data")
        mailByteArrayJson.Headers.ContentDisposition.Name = """json"""
        mailByteArrayJson.Headers.ContentDisposition.FileName = """json.js"""
        Return mailByteArrayJson
    End Function

    'compose mail byte array and file byte array into content
    Private Function GetMultipartFormContent(ByVal emailRequest As EmailRequest, ByVal filename As String, ByVal boundary As String, ByVal fileContents As Byte()) As MultipartFormDataContent
        Dim content = New MultipartFormDataContent(boundary)
        content.Headers.Remove("Content-Type")
        content.Headers.TryAddWithoutValidation("Content-Type", "multipart/form-data; boundary=" & boundary)
        content.Add(GetMailByteArrayContent(emailRequest))
        content.Add(GetFileByteArrayContent(filename, fileContents))
        Return content
    End Function


    Public Async Function GetMailSentReport() As Task(Of String)
        'get authentication code
        Dim authApi As AuthApiWrapper = New AuthApiWrapper()
        Dim authCode As String = Await authApi.GetAuthenticationToken()
        Dim luxSciResponse As LuxSciResponse = Nothing

        'build path and signature
        Dim path As String = userRequestPath & "/" & userName & "/report/email/webmail/sent"
        Dim signature As String = AuthApiWrapper.GetHMACSignature(authCode & vbLf & "GET" & vbLf & path & vbLf & vbLf & vbLf)

        'call HttpClient
        Using userHttpClient As UserHttpClient = New UserHttpClient()
            luxSciResponse = Await userHttpClient.GetEmailSentReport(path, authCode, signature)
        End Using

        'throw exception, if the success is not 1
        If luxSciResponse.success <> 1 Then
            Throw New ApiCallFailedException(luxSciResponse.error_message)
        Else
            Return luxSciResponse.data.ToString()
        End If
    End Function
End Class

